/*
    *
    * This file is a part of CoreShot.
    * A screen capture utility for C Suite.
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include <QFile>
#include <QTimer>
#include <QDateTime>
#include <QFileDialog>
#include <QClipboard>
#include <QMimeData>
#include <QScreen>

#include <cprime/cprime.h>
#include <cprime/appopenfunc.h>
#include <cprime/shareit.h>
#include <cprime/messageengine.h>

#include "coreshotdialog.h"
#include "coreshot.h"
#include "ui_coreshot.h"


coreshot::coreshot(QWidget *parent) : QWidget(parent)
  , ui(new Ui::coreshot)
  , smi(new settings)
{
    ui->setupUi(this);

    loadSettings();
    startSetup();

    files = "";
}

coreshot::~coreshot()
{
	delete smi;
    delete ui;
}
/**
 * @brief Setup ui elements
 */
void coreshot::startSetup()
{
    // set window size
    this->resize(800, 500);
    if (uiMode != 0) {
        this->setWindowState(Qt::WindowMaximized);
    }
}

/**
 * @brief Loads application settings
 */
void coreshot::loadSettings()
{
    uiMode = smi->getValue("CoreApps", "UIMode");
    saveLocation = QString(smi->getValue("CoreShot", "SaveLocation"));
    listViewIconSize = smi->getValue("CoreApps", "ListViewIconSize");
}

void coreshot::setPixmap(QPixmap &pix)
{
    ui->shotPreview->setOriginalPixmap(pix);
}

void coreshot::on_save_clicked()
{
    QString fileName = saveLocation;
    if (!CPrime::FileUtils::isDir(fileName)) {
        qDebug() << "func(on_save_clicked) : Error : Invalid saving directory :" << fileName;
        qDebug() << "creating " << saveLocation;
        QDir().mkpath(saveLocation);
    }

    // Remove the last '/' if exists
    fileName = QDir(fileName).path();

    if (fileName.count()) {
        QFile file(fileName + "/Screenshot_" + QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss") + ".png");
        ui->shotPreview->originalPixmap().save(&file, "PNG");
        file.close();
        files = fileName;
        // Function from utilities.cpp
		CPrime::MessageEngine::messageEngine(
					"org.cubocore.CoreShot",
					"CoreShot",
					"Screenshot is saved",
					"You can see the file on\n" + saveLocation
		);

        this->hide();
        // Based on Message engine time how it will stay
        QTimer::singleShot(1000, this, SLOT(close()));
    } else {
		CPrime::MessageEngine::messageEngine(
					"org.cubocore.CoreShot",
					"CoreShot",
					"Screenshot is not saved",
					"Please select a valid directory from CoreGarage."
		);
        return;
    }
}

void coreshot::on_saveAs_clicked()
{
    QString filename = QFileDialog::getSaveFileName(this, tr("Save Screenshot"), saveLocation, tr("Image (*.png)"));

    if(filename.isEmpty())
        return;

    QFile file(filename + ".png");

    ui->shotPreview->originalPixmap().save(&file, "PNG");

    file.close();
    files = filename;
    // Function from utilities.cpp
	CPrime::MessageEngine::messageEngine(
				"org.cubocore.CoreShot",
				"CoreShot",
				"Screenshot Saved",
				"See the file on saved location."
	);

    this->hide();
    // Based on Message engine time how it will stay
    QTimer::singleShot(1000, this, SLOT(close()));
}

void coreshot::on_cancel_clicked()
{
    this->close();
}

void coreshot::on_newShot_clicked()
{
    coreshotdialog *shotD = new coreshotdialog();
    shotD->show();
    this->close();
}

void coreshot::on_openInEditor_clicked()
{
    QString fileName = saveLocation;
    if (!CPrime::FileUtils::isDir(fileName)) {
        qDebug() << "func(on_openInEditor_clicked) : Error : Invalid saving directory :" << fileName;
        qDebug() << "creating " << saveLocation;
        QDir().mkpath(saveLocation);
    }

    // Remove the last '/' if exists
    fileName = QDir(fileName).path();

    if (!fileName.count()) {
		CPrime::MessageEngine::messageEngine(
					"org.cubocore.CoreShot",
					"CoreShot",
					"Save directory does not exist",
					"Please select a valid directory from CoreGarage.\nOpening in editor needs to save the file first."
		);
        return;
    }

    fileName = saveLocation + "/Screenshot_" + QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss") + ".png";
    QFile file(fileName);
    ui->shotPreview->originalPixmap().save(&file, "PNG");
    file.close();
    files = fileName;

    CPrime::AppOpenFunc::defaultAppEngine(CPrime::DefaultAppCategory::ImageEditor, files, "");
}

void coreshot::on_shareit_clicked()
{
    QString fileName = saveLocation;
    if (!CPrime::FileUtils::isDir(fileName)) {
        qDebug() << "func(on_openInEditor_clicked) : Error : Invalid saving directory :" << fileName;
        qDebug() << "creating " << saveLocation;
        QDir().mkpath(saveLocation);
    }

    // Remove the last '/' if exists
    fileName = QDir(fileName).path();

    if (!fileName.count()) {
		CPrime::MessageEngine::messageEngine(
					"org.cubocore.CoreShot",
					"CoreShot",
					"Save directory does not exist",
					"Please select a valid directory from CoreGarage.\nOpening in editor needs to save the file first."
		);
        return;
    }

    fileName = saveLocation + "/Screenshot_" + QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss") + ".png";
    QFile file(fileName);
    ui->shotPreview->originalPixmap().save(&file, "PNG");
    file.close();
    files = fileName;

    if ( !files.isNull() ) {
        ShareIT *t = new ShareIT(QStringList() << files, listViewIconSize, nullptr);
        // Set ShareIT window size
        if (uiMode == 2 )
            t->setFixedSize(QGuiApplication::primaryScreen()->size() * .8 );
        else
            t->resize(500,600);

        t->exec();
    }
}
